package com.ogow.adapter.itemview

import android.annotation.SuppressLint
import android.graphics.Paint
import android.os.Bundle
import android.text.TextUtils
import android.view.View
import android.widget.TextView
import com.facebook.drawee.view.SimpleDraweeView
import com.ogow.activity.R
import com.ogow.basic.utils.OgowDialogHelper
import com.ogow.brandshop.model.CollectBean
import com.ogow.core.arouter.module.app.AppService
import com.ogow.core.arouter.module.app.DialogService
import com.ogow.core.https.MyRequest
import com.ogow.core.https.entity.RbEntity
import com.ogow.core.https.interfaces.IRespondMsg
import com.ogow.lib.list.base.BaseViewHolder
import com.ogow.lib.list.base.Item
import com.ogow.lib.list.interfaces.OnItemSelectStateChangedListener
import com.ogow.libs.utils.drawable.GradientDrawableHelper
import com.ogow.libs.utils.image.ImageUtils
import com.ogow.request.entity.CollectionBean

/**
 * @author qzp
 * @date 2018/1/5 0005
 */
class CollectGoodsViewHolder (itemView : View) : BaseViewHolder(itemView) {

    private var ivImg : SimpleDraweeView = mParentView.findViewById(R.id.iv_img) as SimpleDraweeView
    private var iv_top_img : SimpleDraweeView = mParentView.findViewById(R.id.iv_top_img) as SimpleDraweeView
    private var txtDate : TextView = mParentView.findViewById(R.id.txt_date) as TextView
    private var tvTitle : TextView = mParentView.findViewById(R.id.tv_title) as TextView
    private var tvValid : TextView = mParentView.findViewById(R.id.tv_valid) as TextView
    private var tvPrice : TextView = mParentView.findViewById(R.id.tv_price) as TextView
    private var tvScprice : TextView = mParentView.findViewById(R.id.tv_scprice) as TextView
    private var actionCancel : TextView = mParentView.findViewById(R.id.action_cancel) as TextView

    lateinit var mData : CollectionBean

    lateinit var listener : OnItemSelectStateChangedListener

    var proDialog : OgowDialogHelper = DialogService.dialogProgress(mParentView.context).customLayout()

    fun setOnItemSelectStateChangedListener( listener : OnItemSelectStateChangedListener) {
        this.listener = listener
    }

    init {
        proDialog.dismiss()

        GradientDrawableHelper.getInstance()
                .setSolidColor(R.color.color_ebebeb)
                .setRoundRadius(40)
                .setPaddingVertical(3)
                .setPaddingHorizontal(10)
                .genarete(tvValid)

        GradientDrawableHelper.getInstance()
                .setRoundRadius(40)
                .setSolidColor(R.color.color_f5f5f5)
                .setPaddingVertical(3)
                .setPaddingHorizontal(6)
                .genarete(txtDate)

        actionCancel.setOnClickListener {
            showPrompt()
        }

        mParentView.setOnClickListener {
            val b = Bundle()
            b.putString("goods_id", mData.goods_id)
            AppService.jumpGoodsDetail(b)
        }
    }

    @SuppressLint("SetTextI18n")
    override fun setItem(item: Item?) {
        if( item != null ) {
            mData = item.obj as CollectionBean

            if ( mData.goods != null ) {

                if( mData.show_top ) {
                    txtDate.visibility = View.VISIBLE

                    if( mData.is_month == 1) {
                        txtDate.text = "1个月前的收藏"
                    } else {
                        txtDate.text = "最近1个月内收藏"
                    }

                } else {
                    txtDate.visibility = View.GONE
                }

                if( !TextUtils.isEmpty(mData.goods.is_onsale) && mData.goods.is_onsale == "1") {
                    tvValid.visibility = View.GONE
                } else {
                    tvValid.visibility = View.VISIBLE
                }


                ImageUtils.getInstance().load(ivImg, mData.goods.default_image)


                if (TextUtils.isEmpty(mData.goods.act_image)) {
                    iv_top_img.visibility = View.GONE
                } else {
                    iv_top_img.visibility = View.VISIBLE
                    ImageUtils.getInstance().loadResize(iv_top_img, mData.goods.act_image, 70, 70)
                }

                tvTitle.text = mData.goods.goods_name
                tvPrice.text = "¥${mData.goods.Price}"
                tvScprice.text = "¥${mData.goods.scPrice}"
                tvScprice.paint.flags = Paint.STRIKE_THRU_TEXT_FLAG //中划线
            }
        }
    }

    private fun showPrompt() {
        if( mParentView.context != null) {
            DialogService.dialogCommon(mParentView.context)
                    .setContent("您确定要取消此收藏吗？")
                    .setActionLeftTxt("放弃")
                    .setActionRightTxt("确定")
                    .setActionRightClick(object : OgowDialogHelper.OnDialogClick {
                        override fun onClick(view: View) {
                            doCancelCollect()
                        }
                    })
                    .customLayout()
        }
    }

    private fun doCancelCollect() {

        if(!proDialog.isShowing())
            proDialog.show()

        val params = HashMap<String, String>()
        params.put("id", mData.id)

        MyRequest.request(this, -1, RbEntity.url.COLLECT, params, object : IRespondMsg {
            override fun doRequest(tag: Int, vararg objs: Any) {

            }

            override fun onRequestSuccess(response: RbEntity) {
                if(proDialog.isShowing())
                    proDialog.dismiss()

                val bean = response.data as CollectBean
                if ( bean.isSuccess) {
                    listener.onItemAction(mData.show_top, mPosition)
                }
            }

            override fun onRequestFail(error: RbEntity) {
                if(proDialog.isShowing())
                    proDialog.dismiss()
            }
        }, CollectBean::class.java, false, false)
    }

}