package com.ogow.dialog

import android.content.ClipData
import android.content.ClipboardManager
import android.content.Context
import android.graphics.drawable.GradientDrawable
import android.support.v7.widget.GridLayoutManager
import android.text.TextUtils
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import android.view.WindowManager
import com.ashlikun.adapter.ViewHolder
import com.ashlikun.adapter.recyclerview.CommonAdapter
import com.ashlikun.utils.AppUtils
import com.ogow.activity.R
import com.ogow.activity.process.ShareShadowActivity
import com.ogow.activity.process.ShareShadowCustomActivity
import com.ogow.activity.wxapi.ShareBack
import com.ogow.basic.utils.ActivityUtils
import com.ogow.libs.dialog.OgowBaseDialog
import com.ashlikun.utils.other.DimensUtils
import com.ogow.libs.utils.OgowUtils
import com.ogow.request.entity.ShareParamData
import com.ogow.request.entity.base.KeyAndValus
import com.ogow.view.SharedGoodsImageView
import com.umeng.socialize.ShareAction
import com.umeng.socialize.bean.SHARE_MEDIA
import com.umeng.socialize.media.UMWeb
import kotlinx.android.synthetic.main.dialog_shared.*

/**
 * 作者　　: 李坤
 * 创建时间: 2018/12/19　11:15
 * 邮箱　　：496546144@qq.com
 *
 * 功能介绍：3.0.0的分享对话框
 * 1：先商品详情用的
 *
 *
 * 图文分享处理
 *  1：商品信息图文---> [SharedGoodsImageView.get]
 *
 *
 *  测试数据
 *
SharedTuwenGoodsData data1 = new SharedTuwenGoodsData();
ArrayList arrayList = new ArrayList();
arrayList.add("买点11");
arrayList.add("买点2");
arrayList.add("买点3");
arrayList.add("买点4444");
data1.setMaidianData(arrayList);
data1.setTitle("我是标题我是标题我是标题我是标题");
data1.setPrice("19.90");
data1.setPriceOld("29.90");
data1.setImageUrl("https://ojn98tluc.qnssl.com/pic0gow_5b5ad629c7e08?imageMogr2/thumbnail/800x/strip/quality/75/format/jpg");
SharedDialog.show(this, data.data, SharedGoodsImageView.get(data1));
 */
class SharedDialog @JvmOverloads constructor(context: Context, themeResId: Int = R.style.Dialog_Botton_Form_top)
    : OgowBaseDialog(context, themeResId), ShareBack.UtilsCallback {
    /**
     * 分享的参数数据
     */
    var sharedParam: ShareParamData? = null
    /**
     * 图文点击回调出去，方便扩展
     */
    var onTwClickListener: ((Context, ShareParamData) -> Unit)? = null
    /**
     * 分享结果的回调监听
     */
    var resultCallback: ((isSuccess: Boolean, isCancle: Boolean) -> Unit)? = null

    companion object {
        //微信
        const val TYPE_WX = 1
        //朋友圈
        const val TYPE_WX_PYQ = 2
        //图文分享
        const val TYPE_TW = 3
        //复制链接
        const val TYPE_RUL = 4

        @JvmStatic
        @JvmOverloads
        fun createDialog(context: Context,
                         sharedParam: ShareParamData,
                         onTwClickListener: ((Context, ShareParamData) -> Unit)? = null,
                         resultCallback: ((isSuccess: Boolean, isCancle: Boolean) -> Unit)? = null): SharedDialog {
            var dialog = SharedDialog(context)
            dialog.sharedParam = sharedParam
            dialog.onTwClickListener = onTwClickListener
            dialog.resultCallback = resultCallback

            return dialog
        }

        /**
         * 显示一个对话框
         * @param sharedParam 基本的分享数据
         * @param onTwClickListener 图文点击事件
         * @param resultCallback 分享结果事件
         */
        @JvmStatic
        @JvmOverloads
        fun show(context: Context,
                 sharedParam: ShareParamData,
                 onTwClickListener: ((Context, ShareParamData) -> Unit)? = null,
                 resultCallback: ((isSuccess: Boolean, isCancle: Boolean) -> Unit)? = null) {
            createDialog(context, sharedParam, onTwClickListener, resultCallback).show()
        }
    }

    private var itemList = ArrayList<KeyAndValus>()

    override fun getLayoutId() = R.layout.dialog_shared
    override fun initView() {
        val drawable = GradientDrawable()
        drawable.setColor(context.resources.getColor(R.color.white))
        var radaii = DimensUtils.dip2px(context, 10f).toFloat()
        drawable.cornerRadii = floatArrayOf(radaii, radaii, radaii, radaii, 0f, 0f, 0f, 0f)
        rootView.background = drawable
        itemList.add(KeyAndValus(R.drawable.icon_shared_wx, valus = "微信", tempKey = TYPE_WX))
        itemList.add(KeyAndValus(R.drawable.icon_shared_pyq, valus = "朋友圈", tempKey = TYPE_WX_PYQ))
        itemList.add(KeyAndValus(R.drawable.icon_shared_tuwen, valus = "图文分享", tempKey = TYPE_TW))
        itemList.add(KeyAndValus(R.drawable.icon_shared_fuzhi, valus = "复制链接", tempKey = TYPE_RUL))

        cancelButton.setOnClickListener {
            dismiss()
        }
        recycleView.layoutManager = GridLayoutManager(context, 4)
        recycleView.adapter = createAdapter()
    }

    override fun initWindowParams(params: WindowManager.LayoutParams) {
        super.initWindowParams(params)
        params.gravity = Gravity.BOTTOM
    }

    override fun getLayoutWidth(): Int {
        return ViewGroup.LayoutParams.MATCH_PARENT
    }

    private fun createAdapter() = object : CommonAdapter<KeyAndValus>(context, R.layout.item_shared_dialog, itemList) {
        override fun convert(holder: ViewHolder, t: KeyAndValus) {
            holder.setText(R.id.nameTv, t.valus)
            holder.setImageResource(R.id.imageView, t.key)
            holder.setVisible(R.id.tuijianIm, t.tempKey == TYPE_TW)
        }

        override fun onItemClick(viewType: Int, parent: ViewGroup, view: View, data: KeyAndValus, position: Int) {
            if (getActivity().isFinishing) {
                return
            }
            sharedParam?.run {
                when (data.tempKey) {
                    TYPE_WX -> wxShared(this, SHARE_MEDIA.WEIXIN)
                    TYPE_WX_PYQ -> wxShared(this, SHARE_MEDIA.WEIXIN_CIRCLE)
                    TYPE_TW -> onTwClickListener?.invoke(context, this)
                    TYPE_RUL -> urlShared(this)
                    else -> {
                    }
                }
                finish()
            }
        }
    }

    private fun getActivity() = ActivityUtils.getActivity(context)

    /**
     * 微信和朋友圈分享
     */
    private fun wxShared(sharedParam: ShareParamData, platform: SHARE_MEDIA) {
        val sha = ShareAction(getActivity())
        sha.platform = platform
        /**
         * 20170717, add by qzp
         * 商品详情页的微信分享需要使用单独的url
         */
        val web = UMWeb(sharedParam.wxUrl)
        web.title = sharedParam.kShareParamTitleKey
        web.description = sharedParam.kShareParamTextKey
        web.setThumb(sharedParam.getUMImage(context))
        sha.withMedia(web)
        sha.setCallback(ShareBack(getActivity(), this))
        sha.share()
    }

    /**
     * 复制链接
     */
    private fun urlShared(sharedParam: ShareParamData) {
        if (!TextUtils.isEmpty(sharedParam.kShareParamURLKey)) {
            try {
                val cmb = AppUtils.getApp().getSystemService(Context.CLIPBOARD_SERVICE) as ClipboardManager
                cmb.primaryClip = ClipData.newPlainText(null, sharedParam.kShareParamURLKey)
                OgowUtils.toastShort("已复制到剪切板")
                if (getActivity() is ShareShadowActivity || getActivity() is ShareShadowCustomActivity) {
                    getActivity().finish()
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
    }

    override fun callbackSucess() {
        resultCallback?.invoke(true, false)
    }

    override fun callbackError() {
        resultCallback?.invoke(false, false)
    }

    override fun callbackCancle() {
        resultCallback?.invoke(false, true)
    }

}