package com.ogow.view

import android.Manifest
import android.content.ClipData
import android.content.ClipboardManager
import android.content.Context
import android.graphics.Bitmap
import android.graphics.drawable.BitmapDrawable
import android.util.TypedValue
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import com.ashlikun.adapter.ViewHolder
import com.ashlikun.adapter.recyclerview.CommonAdapter
import com.ashlikun.utils.AppUtils
import com.ashlikun.utils.other.DimensUtils
import com.ashlikun.utils.other.SpannableUtils
import com.ashlikun.utils.ui.BitmapUtil
import com.ashlikun.utils.ui.DrawableUtils
import com.ashlikun.zxing.CodeUtils
import com.facebook.drawee.drawable.RoundedBitmapDrawable
import com.facebook.drawee.generic.RoundingParams
import com.google.android.flexbox.*
import com.ogow.activity.R
import com.ogow.basic.utils.CacheUtils
import com.ogow.dialog.SharedDialog
import com.ogow.dialog.SharedImageLoadDialog
import com.ogow.dialog.SharedTuwenResultDialog
import com.ogow.libs.utils.PermissionsUtils
import com.ogow.libs.utils.ToastUtils
import com.ashlikun.utils.ui.TextDrawable
import com.ogow.libs.utils.image.ImageLoaderListener
import com.ogow.libs.utils.image.ImageUtils
import com.ogow.request.entity.ShareParamData
import com.ogow.request.entity.goods.SharedTuwenGoodsData
import com.ogow.utility.ViewUtils
import kotlinx.android.synthetic.main.view_shared_goods_tuwen.view.*
import java.lang.ref.WeakReference


/**
 * 作者　　: 李坤
 * 创建时间: 2018/12/19　16:41
 * 邮箱　　：496546144@qq.com
 *
 * 功能介绍：图文分享的图片模板
 * @param tagerView :是需要附属的view的名字
 */
class SharedGoodsImageView constructor(context: Context) : LinearLayout(context) {
    /**
     * 整个布局图片的个数
     */
    val MAX_IMAGE = 2
    /**
     * 当前加载完成几个
     */
    var imageLoadCount = 0
    /**
     * 加载成功了是否要保存图片
     */
    var isGoSave = false
    var isFinishLayout = false
    var onLitener: OnListener? = null

    var loadDialog: SharedImageLoadDialog? = null
    private val adapter by lazy {
        createMaidianAdapter()
    }
    var goodsData: SharedTuwenGoodsData? = null
    var sharData: ShareParamData? = null

    init {
        orientation = VERTICAL
        var padding = DimensUtils.dip2px(context, 15f)
        setPadding(padding, padding, padding, padding)
        setBackgroundResource(R.color.white)
        ViewUtils.getInflaterView(context, R.layout.view_shared_goods_tuwen, this)
        var flexboxLayoutManager = FlexboxLayoutManager(context)
        flexboxLayoutManager.alignItems = AlignItems.CENTER
        flexboxLayoutManager.flexDirection = FlexDirection.ROW
        flexboxLayoutManager.flexWrap = FlexWrap.NOWRAP
        flexboxLayoutManager.justifyContent = JustifyContent.SPACE_BETWEEN
        maidianRv.layoutManager = flexboxLayoutManager
        maidianRv.adapter = adapter
        var ogowFlagDraw = TextDrawable()
                .setTextColor(0xfffc0004.toInt())
                .setText("来自零购官网商品")
                .setPadding(5f, 1f, 5f, 1f)
                .setTextSize(14f)
                .setCornerRadiusNew(3f)
                .setStrokeNew(0.7f, 0xfffc0004.toInt())
        ogowFlagTv.text = SpannableUtils.getBuilder("").setDrawable(ogowFlagDraw)
                .create()
        visibility = View.INVISIBLE
        setWillNotDraw(false)
    }


    /**
     * 构建Ui
     * @param sharData 基本的分享数据
     * @param goodsData 图文分享的数据
     */
    fun bindUi(sharData: ShareParamData, goodsData: SharedTuwenGoodsData): SharedGoodsImageView {
        this.goodsData = goodsData
        this.sharData = sharData
        nameTv.text = goodsData.getName()
        titleTv.text = goodsData.title
        priceTv.text = goodsData.getPriceUi()
        adapter.setDatas(goodsData.maidianData!!, true)
        maidianRv.adapter?.notifyDataSetChanged()
        //是否是将军
        if (sharData.isJiangJun) {
            dengjiIv.visibility = View.VISIBLE
        } else {
            dengjiIv.visibility = View.GONE
        }

        //头像
        ImageUtils.getInstance().loadImage(goodsData.getTouxiang(),
                touxiangImage.layoutParams.width, touxiangImage.layoutParams.height,
                MyImageLoaderListener(touxiangImage, this, RoundingParams().setRoundAsCircle(true)))
        //大图
        ImageUtils.getInstance().loadImage(goodsData.imageUrl,
                mainImg.layoutParams.width, mainImg.layoutParams.height,
                MyImageLoaderListener(mainImg, this, RoundingParams()))
        //二维码
        var codeBitmap = CodeUtils.createImage(sharData.wxUrl, codeIv.layoutParams.width, codeIv.layoutParams.height, null)
        if (codeBitmap != null) {
            codeIv.setImageBitmap(codeBitmap)
        }
        return this
    }

    override fun onLayout(changed: Boolean, l: Int, t: Int, r: Int, b: Int) {
        super.onLayout(changed, l, t, r, b)
        isFinishLayout = true
    }

    /**
     * 把当前view保存成图片
     */
    @JvmOverloads
    fun saveImage(onLitener: OnListener? = null) {
        this.onLitener = onLitener
        isGoSave = true
        loadDialog = SharedImageLoadDialog(context, this)
        loadDialog?.show()
        goSave()
    }

    /**
     * 图片加载成功了，去保存
     */
    private fun goSave() {
        if (imageLoadCount >= MAX_IMAGE) {
            if (isGoSave) {
                if (isFinishLayout) {
                    var isSuccess = false
                    if (PermissionsUtils.hasSelfPermissions(context,
                                    Manifest.permission.READ_EXTERNAL_STORAGE, Manifest.permission.WRITE_EXTERNAL_STORAGE)) {
                        var bitmap = BitmapUtil.getViewBitmap(this)
                        if (bitmap != null) {
                            isSuccess = BitmapUtil.saveImageToGallery(context, bitmap,CacheUtils.newImage())
                        }
                        loadDialog?.dismiss()
                        //显示结果对话框
                        if (isSuccess) {
                            //复制文案到剪切板
                            try {
                                val cmb = AppUtils.getApp().getSystemService(Context.CLIPBOARD_SERVICE) as ClipboardManager
                                cmb.primaryClip = ClipData.newPlainText(null,
                                        goodsData?.getCopyWenan(sharData?.wxUrl ?: "") ?: "")
                            } catch (e: Exception) {

                            }
                            SharedTuwenResultDialog(context).show()
                        } else {
                            ToastUtils.toastShort("图片保存失败")
                        }
                    } else {
                        loadDialog?.dismiss()
                        ToastUtils.toastShort("请开启存储权限")
                    }
                    onLitener?.onSaveResult(isSuccess)
                }
            } else {
                postDelayed({ goSave() }, 100)
            }
        }
    }

    private fun createMaidianAdapter() = object : CommonAdapter<String>(context, null) {

        override fun getItemLayout(parent: ViewGroup, layoutId: Int): View {
            var tv = TextView(context)
            tv.setTextSize(TypedValue.COMPLEX_UNIT_DIP, 13f)
            tv.setTextColor(resources.getColor(R.color.color_999999))
            DrawableUtils.createTextDraw(tv, R.drawable.icon_shared_goods_maidian)
                    .location(1)
                    .heightDp(4)
                    .widthDp(4)
                    .set()
            tv.compoundDrawablePadding = DimensUtils.dip2px(context, 4f)
            tv.maxLines = 1
            return tv
        }

        override fun convert(holder: ViewHolder, t: String) {
            var tv = holder.itemView as TextView
            tv.text = "$t"
        }
    }

    class MyImageLoaderListener : ImageLoaderListener {
        var image: WeakReference<ImageView?>? = null
        var roundParams: RoundingParams? = null
        var rootView: WeakReference<SharedGoodsImageView?>? = null

        constructor(image: ImageView, view: SharedGoodsImageView, roundParams: RoundingParams?) {
            this.image = WeakReference(image)
            this.rootView = WeakReference(view)
            this.roundParams = roundParams
        }

        override fun onLoadFailed(throwable: Throwable?) {
            image?.get()?.run {
                setImageResource(R.drawable.loading_image)
            }
            rootView?.get()?.run {
                imageLoadCount++
                goSave()
            }
        }

        override fun onLoadComplete(bitmap: Bitmap) {
            image?.get()?.run {
                var bitDraw = BitmapDrawable()
                val roundedBitmapDrawable = RoundedBitmapDrawable(
                        resources,
                        bitmap,
                        bitDraw.paint)
                roundedBitmapDrawable.isCircle = roundParams?.roundAsCircle ?: false
                setImageDrawable(roundedBitmapDrawable)
            }
            rootView?.get()?.run {
                imageLoadCount++
                goSave()
            }
        }
    }

    /**
     * 回调结果
     */
    interface OnListener {
        fun onSaveResult(isSuccess: Boolean)
    }

    companion object {
        /**
         * 创建一个[SharedDialog] 需要的图文点击回调
         */
        @JvmStatic
        fun get(goodsData: SharedTuwenGoodsData): (context: Context, sharData: ShareParamData) -> Unit {
            return { context, sharData ->
                SharedGoodsImageView(context)
                        .bindUi(sharData, goodsData)
                        .saveImage()
            }
        }
    }
}